class Artist < ActiveRecord::Base
  belongs_to :author
  has_many :original_pictures
  has_many :pictures
  has_many :resource_pictures
  
  validates :name, :presence => true, :length => {:maximum => 30}
  validates :author_id, :numericality => {:allow_blank => true}
  
  def supply_default
    self.name = 'no name' if self.name.blank?
  end
  
  def overwrite au
    return false unless au
    self.author_id = au.id
  end
  
  def own? roles
    roles = [roles] unless roles.respond_to?(:each)
    au = Artist.get_author_from_roles roles
    return false unless au
    self.author_id == au.id
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless resource_reader_role_check(roles)
    end
    true
  end
  
  def self.find_by_author author
    Artist.find( :first, :conditions => ['author_id = ?', author.id])
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.offset cnt, prm = nil
    offset = prm.to_i
    offset = cnt - 1 if offset >= cnt
    offset = cnt - offset.abs if offset < 0
    offset = 0 if offset < 0
    offset
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(Artist.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['artists.author_id is not null'], :order => 'artists.created_at desc'})
    Artist.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:author => {}} }
  end
  
  def self.list_json_opt
    {:include => {:author => {}} }
  end
  
  def self.show aid, au
    opt = {}
    opt.merge!(Artist.show_opt)
    res = Artist.find(aid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(au)
    res
  end
  
  def self.edit sid, au
    opt = {}
    opt.merge!(Artist.show_opt)
    res = Artist.find sid, opt
    raise ActiveRecord::Forbidden unless res.own?(au)
    res
  end
  
  def self.show_opt
    {:include => {:author => {}} }
  end
  
  def self.show_json_opt
    {:include => {:author => {}} }
  end
  
  def self.visible_count
    Artist.count :conditions => ['artists.author_id is not null']
  end
  
  def self.export(dt = nil)
    opt = {}
    cond = if dt
      ['artists.author_id is not null and artists.updated_at >= ?', dt]
    else
      'artists.author_id is not null'
    end
    opt.merge!({:conditions => cond}) 
    opt.merge!({:order => 'id'})
    Artist.find(:all, opt)
  end
  
end
