class GroundColor < ActiveRecord::Base
  belongs_to :panel
  belongs_to :color
  
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :color_id, :numericality => true, :existence => {:both => false}
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  
  def supply_default
  end
  
  def overwrite
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.panel.own?(roles)
    self.panel.visible? roles
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(GroundColor.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => 'panels.publish > 0', :order => 'ground_colors.updated_at desc'})
    GroundColor.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:panel => {:author => {}}, :color => {} }}
  end
  
  def self.list_json_opt
    {:include => {:panel => {:include => {:author => {}}}, :color => {} }}
  end
  
  def self.mylist au, page = 1, page_size = Author.default_ground_color_page_size
    opt = {}
    opt.merge!(self.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['panels.author_id = ?', au.id], :order => 'ground_colors.updated_at desc'})
    GroundColor.find(:all, opt)
  end
  
  def self.show cid, roles
    opt = {}
    opt.merge!(GroundColor.show_opt)
    res = GroundColor.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(roles)
    res
  end
  
  def self.show_opt
    {:include => {:panel => {:author => {}}, :color => {} }}
  end
  
  def self.show_json_opt
    {:include => {:panel => {:include => {:author => {}}}, :color => {} }}
  end
  
end
