class Speech < ActiveRecord::Base
  belongs_to :speech_balloon
  
  validates :speech_balloon_id, :numericality => {:allow_blank => true}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
#  validates :settings, :presence => true
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.speech_balloon.panel.own?(roles)
    self.speech_balloon.panel.visible? roles
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(Speech.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => 'panels.publish > 0', :order => 'speeches.updated_at desc'})
    Speech.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:speech_balloon => {:panel => {:author => {}}, :balloons => {}, :speech_balloon_template => {}} }}
  end
  
  def self.list_json_opt
    {:include => {:speech_balloon => {:include => {:panel => {:include => {:author => {} }}, :balloons => {}, :speech_balloon_template => {} }}}}
  end
  
  def self.show cid, au
    opt = {}
    opt.merge!(Speech.show_opt)
    res = Speech.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(au)
    res
  end
  
  def self.show_opt
    {:include => {:speech_balloon => {:panel => {:author => {}}, :balloons => {}, :speech_balloon_template => {} }}}
  end
  
  def self.show_json_opt
    {:include => {:speech_balloon => {:include => {:panel => {:include => {:author => {} }}, :balloons => {}, :speech_balloon_template => {} }}}}
  end
  
end
