# -*- encoding: utf-8 -*-
#貸与状況
require 'spec_helper'

describe DemanderStatus do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create :user_yas
    @author = @user.author    #ユーザ作成時に連動して作成される
    @demand_user = FactoryGirl.create :demand_user
  end
  describe '検証に於いて' do
    before do
      @ds = FactoryGirl.build :demander_status
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @ds.receive_hour1 = -99999
        @ds.receive_hour2 = -99999
        @ds.should be_valid
      end
      it '上限データが通る' do
        @ds.receive_hour1 = 99999
        @ds.receive_hour2 = 99999
        @ds.should be_valid
      end
    end
    context 'requested_atを検証するとき' do
    end
    context 'rejected_atを検証するとき' do
    end
    context 'approved_atを検証するとき' do
    end
    context 'tokenを検証するとき' do
    end
    context 'receive_hour1を検証するとき' do
      it '数値でなければ失敗する' do
        @ds.receive_hour1 = 'a'
        @ds.should_not be_valid
      end
    end
    context 'receive_hour2を検証するとき' do
      it '数値でなければ失敗する' do
        @ds.receive_hour2 = 'a'
        @ds.should_not be_valid
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it 'defined' do
      @ds = FactoryGirl.build :demander_status
      @ds.supply_default
    end
  end
  
  describe '上書き補充に於いて' do
    it 'defined' do
      @ds = FactoryGirl.build :demander_status
      @ds.overwrite
    end
  end
  
  describe '所持判定に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
    end
    it '管理者ならyes' do
      @ds.own?(@admin).should == true
    end
    it '作家ならno' do
      @ds.own?(@author).should == false
    end
    it 'パラメータが管理者でないならno' do
      @ds.own?(nil).should == false
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
    end
    it '管理者なら許可する' do
      r = @ds.visible?(@admin)
      r.should == true
    end
    it '作家なら許可しない' do
      r = @ds.visible?(@author)
      r.should == false
    end
    it 'それ以外のとき不許可を返す。' do
      r = @ds.visible?(nil)
      r.should be_false
    end
  end
  
  describe '状態に於いて' do
    before do
    end
    context '申請日時がセットされていないとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => nil, :approved_at => Time.now
      end
      it '待機中0を返す' do
        r = @ds.status
        r.should eq 0
      end
    end
    context '申請日時と否決日時がセットされているとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :rejected_at => Time.now
      end
      it '否決 1を返す' do
        r = @ds.status
        r.should eq 1
      end
    end
    context '申請日時がセットされていて、可決日時と否決日時がセットされていないとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
      end
      it '申請中 2を返す' do
        r = @ds.status
        r.should eq 2
      end
    end
    context '申請日時と可決日時がセットされているとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => Time.now
      end
      it '貸与中 3を返す' do
        r = @ds.status
        r.should eq 3
      end
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id, :name => "6"
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        DemanderStatus.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        DemanderStatus.page().should eq 1
      end
      it '0以下の場合は1になる' do
        DemanderStatus.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        DemanderStatus.page_size('7').should eq 7
      end
      it 'nilの場合はDemanderStatus.default_page_sizeになる' do
        DemanderStatus.page_size().should eq DemanderStatus.default_page_size
      end
      it '0以下の場合はDemanderStatus.default_page_sizeになる' do
        DemanderStatus.page_size('0').should eq DemanderStatus.default_page_size
      end
      it 'DemanderStatus.max_page_sizeを超えた場合はDemanderStatus.max_page_sizeになる' do
        DemanderStatus.page_size('1000').should eq DemanderStatus.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        DemanderStatus.stub(:list_opt).with(any_args).and_return({:include => {:demander => {}} })
        DemanderStatus.should_receive(:list_opt).with(any_args).exactly(1)
        r = DemanderStatus.list
      end
    end
    it 'リストを返す' do
      r = DemanderStatus.list
      r.should eq [@ds]
    end
    it '管理名で並んでいる' do
      v = FactoryGirl.create :demander_status
      vd = FactoryGirl.create :demander, :demander_status_id => v.id, :demand_user_id => @demand_user.id, :name => "0"
      r = DemanderStatus.list
      r.should eq [v, @ds]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @ds2 = FactoryGirl.create :demander_status
        @demander2 = FactoryGirl.create :demander, :demander_status_id => @ds2.id, :demand_user_id => @demand_user.id, :name => "5"
        @ds3 = FactoryGirl.create :demander_status
        @demander3 = FactoryGirl.create :demander, :demander_status_id => @ds3.id, :demand_user_id => @demand_user.id, :name => "4"
        @ds4 = FactoryGirl.create :demander_status
        @demander4 = FactoryGirl.create :demander, :demander_status_id => @ds4.id, :demand_user_id => @demand_user.id, :name => "3"
        @ds5 = FactoryGirl.create :demander_status
        @demander5 = FactoryGirl.create :demander, :demander_status_id => @ds5.id, :demand_user_id => @demand_user.id, :name => "2"
        DemanderStatus.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        r = DemanderStatus.list
        r.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #管理名で並んでいる
        r = DemanderStatus.list(1)
        r.should eq [@ds5, @ds4]
      end
      it 'page=2なら中間2件を返す' do
        r = DemanderStatus.list(2)
        r.should eq [@ds3, @ds2]
      end
      it 'page=3なら先頭1件を返す' do
        r = DemanderStatus.list(3)
        r.should eq [@ds]
      end
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @ds2 = FactoryGirl.create :demander_status
        @demander2 = FactoryGirl.create :demander, :demander_status_id => @ds2.id, :demand_user_id => @demand_user.id, :name => "5"
        @ds3 = FactoryGirl.create :demander_status
        @demander3 = FactoryGirl.create :demander, :demander_status_id => @ds3.id, :demand_user_id => @demand_user.id, :name => "4"
        @ds4 = FactoryGirl.create :demander_status
        @demander4 = FactoryGirl.create :demander, :demander_status_id => @ds4.id, :demand_user_id => @demand_user.id, :name => "3"
        @ds5 = FactoryGirl.create :demander_status
        @demander5 = FactoryGirl.create :demander, :demander_status_id => @ds5.id, :demand_user_id => @demand_user.id, :name => "2"
        DemanderStatus.stub(:default_page_size).and_return(2)
      end
      it '件数0は全件(5件)を返す' do
        r = DemanderStatus.list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe '申請中一覧取得に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id, :name => "6"
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        DemanderStatus.stub(:list_opt).with(any_args).and_return({:include => {:demander => {}} })
        DemanderStatus.should_receive(:list_opt).with(any_args).exactly(1)
        r = DemanderStatus.requested_list
      end
    end
    it 'リストを返す' do
      r = DemanderStatus.requested_list
      r.should eq [@ds]
    end
    it '管理名で並んでいる' do
      v = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
      vd = FactoryGirl.create :demander, :demander_status_id => v.id, :demand_user_id => @demand_user.id, :name => "0"
      r = DemanderStatus.requested_list
      r.should eq [v, @ds]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @ds2 = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
        @demander2 = FactoryGirl.create :demander, :demander_status_id => @ds2.id, :demand_user_id => @demand_user.id, :name => "5"
        @ds3 = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
        @demander3 = FactoryGirl.create :demander, :demander_status_id => @ds3.id, :demand_user_id => @demand_user.id, :name => "4"
        @ds4 = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
        @demander4 = FactoryGirl.create :demander, :demander_status_id => @ds4.id, :demand_user_id => @demand_user.id, :name => "3"
        @ds5 = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
        @demander5 = FactoryGirl.create :demander, :demander_status_id => @ds5.id, :demand_user_id => @demand_user.id, :name => "2"
        DemanderStatus.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        r = DemanderStatus.requested_list
        r.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #管理名で並んでいる
        r = DemanderStatus.requested_list(1)
        r.should eq [@ds5, @ds4]
      end
      it 'page=2なら中間2件を返す' do
        r = DemanderStatus.requested_list(2)
        r.should eq [@ds3, @ds2]
      end
      it 'page=3なら先頭1件を返す' do
        r = DemanderStatus.requested_list(3)
        r.should eq [@ds]
      end
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @ds2 = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
        @demander2 = FactoryGirl.create :demander, :demander_status_id => @ds2.id, :demand_user_id => @demand_user.id, :name => "5"
        @ds3 = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
        @demander3 = FactoryGirl.create :demander, :demander_status_id => @ds3.id, :demand_user_id => @demand_user.id, :name => "4"
        @ds4 = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
        @demander4 = FactoryGirl.create :demander, :demander_status_id => @ds4.id, :demand_user_id => @demand_user.id, :name => "3"
        @ds5 = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => nil, :rejected_at => nil
        @demander5 = FactoryGirl.create :demander, :demander_status_id => @ds5.id, :demand_user_id => @demand_user.id, :name => "2"
        DemanderStatus.stub(:default_page_size).and_return(2)
      end
      it '件数0は全件(5件)を返す' do
        r = DemanderStatus.requested_list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe '一覧取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = DemanderStatus.list_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = DemanderStatus.list_opt[:include]
      r.should have(1).items
    end
    it '借手を含んでいる' do
      r = DemanderStatus.list_opt[:include]
      r.has_key?(:demander).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
    end
    it '借手を含んでいる' do
      r = DemanderStatus.list.to_json DemanderStatus.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('demander').should be_true
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        DemanderStatus.stub(:show_opt).with(any_args).and_return({})
        DemanderStatus.should_receive(:show_opt).with(any_args).exactly(1)
        r = DemanderStatus.show @ds.id, @admin
      end
      it '閲覧許可を問い合わせている' do
        DemanderStatus.any_instance.stub(:visible?).with(any_args).and_return(true)
        DemanderStatus.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = DemanderStatus.show @ds.id, @admin
      end
    end
    it '指定の貸与状況を返す' do
      r = DemanderStatus.show @ds.id, @admin
      r.should eq @ds
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        DemanderStatus.any_instance.stub(:visible?).and_return(false)
        lambda{
          DemanderStatus.show @ds.id, @admin
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しない貸与状況を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          DemanderStatus.show 110, @admin
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  
  describe '編集取得に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        DemanderStatus.stub(:show_opt).with(any_args).and_return({})
        DemanderStatus.should_receive(:show_opt).with(any_args).exactly(1)
        r = DemanderStatus.edit @ds.id, @admin
      end
      it '所持判定を問い合わせている' do
        DemanderStatus.any_instance.stub(:own?).with(any_args).and_return(true)
        DemanderStatus.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = DemanderStatus.edit @ds.id, @admin
      end
    end
    it '指定の貸与状況を返す' do
      DemanderStatus.any_instance.stub(:own?).and_return(true)
      r = DemanderStatus.edit @ds.id, @admin
      r.should eq @ds
    end
    context '権限がなかったとき' do
      it '403Forbidden例外を返す' do
        DemanderStatus.any_instance.stub(:own?).and_return(false)
        lambda{
          DemanderStatus.edit @ds.id, @admin
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しない貸与状況を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          DemanderStatus.edit 110, @admin
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = DemanderStatus.show_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = DemanderStatus.show_opt[:include]
      r.should have(1).items
    end
    it '借手を含んでいる' do
      r = DemanderStatus.show_opt[:include]
      r.has_key?(:demander).should be_true
    end
  end
  describe 'json単体出力オプションに於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
    end
    it '借手を含んでいる' do
      r = DemanderStatus.show(@ds.id, @admin).to_json DemanderStatus.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('demander').should be_true
    end
  end
  
  describe '借受申請に於いて' do
    before do
    end
    context '待機中状態がつつがなく終わるとき' do
      before do
        @ds = FactoryGirl.create :demander_status
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it '否決日時をクリアする' do
        r = @ds.request
        @ds.rejected_at.should be_nil
      end
      it '可決日時をクリアする' do
        r = @ds.request
        @ds.approved_at.should be_nil
      end
      it '申請日時に現在時刻をセットする' do
        r = @ds.request
        @ds.requested_at.should_not be_nil
      end
      it 'Trueを返す' do
        r = @ds.request
        r.should be_true
      end
    end
    context '否決状態がつつがなく終わるとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :rejected_at => Time.now
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it '否決日時をクリアする' do
        r = @ds.request
        @ds.rejected_at.should be_nil
      end
      it '可決日時をクリアする' do
        r = @ds.request
        @ds.approved_at.should be_nil
      end
      it '申請日時に現在時刻をセットする' do
        r = @ds.request
        @ds.requested_at.should_not be_nil
      end
      it 'Trueを返す' do
        r = @ds.request
        r.should be_true
      end
    end
    context '状態が申請中のとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it 'Falseを返す' do
        r = @ds.request
        r.should be_false
      end
      it '申請中2を返す' do
        r = @ds.request
        @ds.status.should eq 2
      end
    end
    context '状態が貸与中のとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => Time.now
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it 'Falseを返す' do
        r = @ds.request
        r.should be_false
      end
      it '貸与中3を返す' do
        r = @ds.request
        @ds.status.should eq 3
      end
    end
  end
  
  describe '貸与停止に於いて' do
    before do
    end
    context '状態が待機中のとき' do
      before do
        @ds = FactoryGirl.create :demander_status
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it 'Falseを返す' do
        r = @ds.reject
        r.should be_false
      end
      it '待機中0を返す' do
        r = @ds.reject
        @ds.status.should eq 0
      end
    end
    context '状態が否決中のとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :rejected_at => Time.now
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it 'Falseを返す' do
        r = @ds.reject
        r.should be_false
      end
      it '否決中1を返す' do
        r = @ds.reject
        @ds.status.should eq 1
      end
    end
    context '申請中の状態がつつがなく終わるとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it '否決日時をセットする' do
        r = @ds.reject
        @ds.rejected_at.should_not be_nil
      end
      it '可決日時をクリアする' do
        r = @ds.reject
        @ds.approved_at.should be_nil
      end
      it '借手の認証トークンをクリアする' do
        DemandUser.any_instance.stub(:delete_token).with(any_args).and_return(true)
        DemandUser.any_instance.should_receive(:delete_token).with(any_args).exactly(1)
        r = @ds.reject
      end
      it 'Trueを返す' do
        r = @ds.reject
        r.should be_true
      end
    end
    context '状態が貸与中のとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => Time.now
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it '否決日時をセットする' do
        r = @ds.reject
        @ds.rejected_at.should_not be_nil
      end
      it '可決日時をクリアする' do
        r = @ds.reject
        @ds.approved_at.should be_nil
      end
      it '借手の認証トークンをクリアする' do
        DemandUser.any_instance.stub(:delete_token).with(any_args).and_return(true)
        DemandUser.any_instance.should_receive(:delete_token).with(any_args).exactly(1)
        r = @ds.reject
      end
      it 'Trueを返す' do
        r = @ds.reject
        r.should be_true
      end
    end
  end
  
  describe '貸与承認に於いて' do
    before do
    end
    context '状態が待機中のとき' do
      before do
        @ds = FactoryGirl.create :demander_status
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it 'Falseを返す' do
        r = @ds.approve 1,13
        r.should be_false
      end
      it '待機中0を返す' do
        r = @ds.approve 1,13
        @ds.status.should eq 0
      end
    end
    context '状態が否決中のとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :rejected_at => Time.now
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it '可決日時をセットする' do
        r = @ds.approve 1,13
        @ds.approved_at.should_not be_nil
      end
      it '巡回時1と巡回時2をセットする' do
        r = @ds.approve 1,13
        @ds.receive_hour1.should_not be_nil
        @ds.receive_hour2.should_not be_nil
      end
      it '否決日時をクリアする' do
        r = @ds.approve 1,13
        @ds.rejected_at.should be_nil
      end
      it '借手の認証トークンをセットする' do
        DemandUser.any_instance.stub(:create_token).with(any_args).and_return(true)
        DemandUser.any_instance.should_receive(:create_token).with(any_args).exactly(1)
        r = @ds.approve 1,13
      end
      it 'Trueを返す' do
        r = @ds.approve 1,13
        r.should be_true
      end
    end
    context '申請中の状態がつつがなく終わるとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it '可決日時をセットする' do
        r = @ds.approve 1,13
        @ds.approved_at.should_not be_nil
      end
      it '巡回時1と巡回時2をセットする' do
        r = @ds.approve 1,13
        @ds.receive_hour1.should_not be_nil
        @ds.receive_hour2.should_not be_nil
      end
      it '否決日時をクリアする' do
        r = @ds.approve 1,13
        @ds.rejected_at.should be_nil
      end
      it '借手の認証トークンをセットする' do
        DemandUser.any_instance.stub(:create_token).with(any_args).and_return(true)
        DemandUser.any_instance.should_receive(:create_token).with(any_args).exactly(1)
        r = @ds.approve 1,13
      end
      it 'Trueを返す' do
        r = @ds.approve 1,13
        r.should be_true
      end
    end
    context '状態が貸与中のとき' do
      before do
        @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => Time.now
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
      end
      it 'Falseを返す' do
        r = @ds.approve 1,13
        r.should be_false
      end
      it '貸与中3を返す' do
        r = @ds.approve 1,13
        @ds.status.should eq 3
      end
    end
  end
  
  describe '借受停止に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => Time.now, :rejected_at => Time.now, :receive_hour1 => 0, :receive_hour2 => 0
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
    end
    context '事前チェックする' do
      it '借手の認証トークンにクリアを依頼している' do
        DemandUser.any_instance.stub(:delete_token).with(any_args).and_return(true)
        DemandUser.any_instance.should_receive(:delete_token).with(any_args).exactly(1)
        r = @ds.stop
      end
    end
    context 'つつがなく終わるとき' do
      before do
        DemandUser.any_instance.stub(:delete_token).with(any_args).and_return(true)
      end
      it '申請日時をクリアする' do
        r = @ds.stop
        @ds.approved_at.should be_nil
      end
      it '可決日時をクリアする' do
        r = @ds.stop
        @ds.approved_at.should be_nil
      end
      it '否決日時をクリアする' do
        r = @ds.stop
        @ds.rejected_at.should be_nil
      end
      it '巡回時1と巡回時2をクリアする' do
        r = @ds.stop
        @ds.receive_hour1.should be_nil
        @ds.receive_hour2.should be_nil
      end
      it 'Trueを返す' do
        r = @ds.stop
        r.should be_true
      end
    end
    context '自身の保存に失敗したとき' do
      before do
        DemanderStatus.any_instance.stub(:save).with(any_args).and_return(false)
      end
      it 'ロールバックする' do
        r = @ds.stop
        @ds.reload
        @ds.requested_at.should_not be_nil
        @ds.approved_at.should_not be_nil
        @ds.receive_hour2.should_not be_nil
      end
      it 'Falseを返す' do
        r = @ds.stop
        r.should be_false
      end
    end
    context '借手アカウントの保存に失敗したとき' do
      before do
        DemandUser.any_instance.stub(:delete_token).with(any_args).and_return(false)
      end
      it 'ロールバックする' do
        r = @ds.stop
        @ds.reload
        @ds.requested_at.should_not be_nil
        @ds.approved_at.should_not be_nil
        @ds.receive_hour2.should_not be_nil
      end
      it 'Falseを返す' do
        r = @ds.stop
        r.should be_false
      end
    end
  end
end

